// Copyright 2007 "Gilles Degottex"

// This file is part of "Music"

// "Music" is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// "Music" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _Note_h_
#define _Note_h_

#include <QString>
#include <deque>
using namespace std;

class Note
{
  public:
	int ht;
	QString factor;

	// error to ht
	float min_err;
	float max_err;
	float avg_err;
	deque<float> errors;

	// volume
	deque<float> volumes;

	// ctor
	void init();
	Note(int h);
	Note(int h, int num, int den);
	Note(int h, float cents);

	void addAnalysis(float err, float volume);
	QString getName() const;
};

#endif
